@@ -80,7 +80,133 @@ void remove_mskprefix(string &str, string prefix)
 	}
 }
 
-void append_initsol(MSKtask_t task, SEXP_LIST initsol, MSKintt NUMCON, MSKintt NUMVAR)
+bool isdef_solitem(MSKsoltypee s, MSKsoliteme v, bool isGeneralNLP)
+{
+	switch (v)
+	{
+		// Primal variables
+		case MSK_SOL_ITEM_XC:
+		case MSK_SOL_ITEM_XX:
+			switch (s) {
+				case MSK_SOL_ITR:  return true;
+				case MSK_SOL_BAS:  return true;
+				case MSK_SOL_ITG:  return true;
+				default:
+					throw msk_exception("A solution type was not supported");
+			} break;
+
+		// Linear dual variables
+		case MSK_SOL_ITEM_SLC:
+		case MSK_SOL_ITEM_SLX:
+		case MSK_SOL_ITEM_SUC:
+		case MSK_SOL_ITEM_SUX:
+			switch (s) {
+				case MSK_SOL_ITR:  return true;
+				case MSK_SOL_BAS:  return true;
+				case MSK_SOL_ITG:  return false;
+				default:
+					throw msk_exception("A solution type was not supported");
+			} break;
+
+		// Conic dual variable
+		case MSK_SOL_ITEM_SNX:
+			switch (s) {
+				case MSK_SOL_ITR:  return !isGeneralNLP;  // No conic information in a general non-linear programming solution
+				case MSK_SOL_BAS:  return false;
+				case MSK_SOL_ITG:  return false;
+				default:
+					throw msk_exception("A solution type was not supported");
+			} break;
+
+		// Ignored variables
+		case MSK_SOL_ITEM_Y:
+			return false;
+			break;
+
+		// Unsupported variables
+		default:
+			throw msk_exception("A solution item was not supported");
+	}
+}
+
+void getspecs_soltype(MSKsoltypee stype, string &name)
+{
+	switch (stype) {
+		case MSK_SOL_BAS:
+			name = "bas";
+			break;
+		case MSK_SOL_ITR:
+			name = "itr";
+			break;
+		case MSK_SOL_ITG:
+			name = "int";
+			break;
+		default:
+			throw msk_exception("A solution type was not supported");
+	}
+}
+
+void getspecs_sk_solitem(MSK_sk_soliteme vtype, MSKintt NUMVAR, MSKintt NUMCON, MSKintt NUMCONES, string &name, MSKidxt &size)
+{
+	switch (vtype) {
+		case MSK_SK_SOL_ITEM_SKC:
+			name = "skc";
+			size = NUMCON;
+			break;
+		case MSK_SK_SOL_ITEM_SKX:
+			name = "skx";
+			size = NUMVAR;
+			break;
+		case MSK_SK_SOL_ITEM_SKN:
+			name = "skn";
+			size = NUMCONES;
+			break;
+		default:
+			throw msk_exception("A solution status key item was not supported");
+	}
+}
+
+void getspecs_solitem(MSKsoliteme vtype, MSKintt NUMVAR, MSKintt NUMCON, string &name, MSKidxt &size)
+{
+	switch (vtype) {
+		case MSK_SOL_ITEM_XC:
+			name = "xc";
+			size = NUMCON;
+			break;
+		case MSK_SOL_ITEM_XX:
+			name = "xx";
+			size = NUMVAR;
+			break;
+		case MSK_SOL_ITEM_Y:
+			name = "y";
+			size = NUMCON;
+			break;
+		case MSK_SOL_ITEM_SLC:
+			name = "slc";
+			size = NUMCON;
+			break;
+		case MSK_SOL_ITEM_SUC:
+			name = "suc";
+			size = NUMCON;
+			break;
+		case MSK_SOL_ITEM_SLX:
+			name = "slx";
+			size = NUMVAR;
+			break;
+		case MSK_SOL_ITEM_SUX:
+			name = "sux";
+			size = NUMVAR;
+			break;
+		case MSK_SOL_ITEM_SNX:
+			name = "snx";
+			size = NUMVAR;
+			break;
+		default:
+			throw msk_exception("A solution item was not supported");
+	}
+}
+
+void append_initsol(MSKtask_t task, SEXP_LIST initsol, MSKintt NUMCON, MSKintt NUMVAR, MSKintt NUMCONES)
 {
 	// VECTOR_ELT and STRING_ELT needs type 'int'
 	R_len_t numobjects = numeric_cast<int>(Rf_length(initsol));
@@ -118,131 +244,84 @@ void append_initsol(MSKtask_t task, SEXP_LIST initsol, MSKintt NUMCON, MSKintt N
 		}
 		SEXP_LIST cursol = val;
 
-		// Get current solution items
-		SEXP_Handle skc; list_seek_Character(skc, cursol, "skc", true);	validate_Character(skc, "skc", NUMCON, true);
-		SEXP_Handle xc;  list_seek_Numeric(xc, cursol, "xc", true);		validate_Numeric(xc, "xc", NUMCON, true);
-		SEXP_Handle slc; list_seek_Numeric(slc, cursol, "slc", true);	validate_Numeric(slc, "slc", NUMCON, true);
-		SEXP_Handle suc; list_seek_Numeric(suc, cursol, "suc", true);	validate_Numeric(suc, "suc", NUMCON, true);
+		bool any_info = false;
 
-		SEXP_Handle skx; list_seek_Character(skx, cursol, "skx", true);	validate_Character(skx, "skx", NUMVAR, true);
-		SEXP_Handle xx;  list_seek_Numeric(xx, cursol, "xx", true);		validate_Numeric(xx, "xx", NUMVAR, true);
-		SEXP_Handle slx; list_seek_Numeric(slx, cursol, "slx", true);	validate_Numeric(slx, "slx", NUMVAR, true);
-		SEXP_Handle sux; list_seek_Numeric(sux, cursol, "sux", true);	validate_Numeric(sux, "sux", NUMVAR, true);
-		SEXP_Handle snx; list_seek_Numeric(snx, cursol, "snx", true);	validate_Numeric(snx, "snx", NUMVAR, true);
+		MSKstakeye *solitem_sk[MSK_SK_SOL_ITEM_END];
+		auto_array<MSKstakeye> solitem_sk_bank[MSK_SK_SOL_ITEM_END];
+		{
+			string name; MSKidxt size;
 
-		bool anyinfocon = !isEmpty(skc) || !isEmpty(xc) || !isEmpty(slc) || !isEmpty(suc);
-		bool anyinfovar = !isEmpty(skx) || !isEmpty(xx) || !isEmpty(slx) || !isEmpty(sux) || !isEmpty(snx);
+			for (int i = MSK_SK_SOL_ITEM_BEGIN; i < MSK_SK_SOL_ITEM_END; ++i) {
+				MSK_sk_soliteme itype = static_cast<MSK_sk_soliteme>(i);
+				solitem_sk[i] = NULL;
 
-		// Set all constraints
-		if (anyinfocon) {
-			for (MSKintt ci = 0; ci < NUMCON; ci++)
-			{
-				MSKstakeye curskc;
-				if (isEmpty(skc) || CHARACTER_ELT(skc,ci).empty()) {
-					curskc = MSK_SK_UNK;
-				} else {
-					MSKintt skcval;
-					errcatch( MSK_strtosk(task, const_cast<MSKCONST char*>(CHARACTER_ELT(skc,ci).c_str()), &skcval) );
-					curskc = static_cast<MSKstakeye>(skcval);
-				}
+				SEXP_Handle skvalue;
+				getspecs_sk_solitem(itype, NUMVAR, NUMCON, NUMCONES, name, size);
+				list_seek_Character(skvalue, cursol, name, true);  validate_Character(skvalue, name, size, true);
 
-				MSKrealt curxc;
-				if (isEmpty(xc)) {
-					curxc = 0.0;
-				} else {
-					curxc = NUMERIC_ELT(xc, ci);
-				}
+				if (!isEmpty(skvalue)) {
+					any_info = true;
 
-				MSKrealt curslc;
-				if (isEmpty(slc)) {
-					curslc = 0.0;
-				} else {
-					curslc = NUMERIC_ELT(slc, ci);
-				}
+					solitem_sk_bank[i].protect(new MSKstakeye[size]);
+					solitem_sk[i] = solitem_sk_bank[i];
 
-				MSKrealt cursuc;
-				if (isEmpty(suc)) {
-					cursuc = 0.0;
-				} else {
-					cursuc = NUMERIC_ELT(suc, ci);
-				}
-
-				errcatch( MSK_putsolutioni(task,
-						MSK_ACC_CON, ci, cursoltype,
-						curskc, curxc, curslc, cursuc, 0.0));
-			}
-		}
-
-		// Set all variables
-		if (anyinfovar) {
-			for (MSKintt xi = 0; xi < NUMVAR; xi++)
-			{
-				MSKstakeye curskx;
-				if (isEmpty(skx) || CHARACTER_ELT(skx,xi).empty()) {
-					curskx = MSK_SK_UNK;
-				} else {
-					MSKintt skxval;
-					errcatch( MSK_strtosk(task, const_cast<MSKCONST char*>(CHARACTER_ELT(skx,xi).c_str()), &skxval) );
-					curskx = static_cast<MSKstakeye>(skxval);
-				}
-
-				MSKrealt curxx;
-				if (isEmpty(xx)) {
-
-					// Variable not fully defined => use bound information
-					MSKboundkeye bk;
-					MSKrealt bl,bu;
-					errcatch( MSK_getbound(task,
-							MSK_ACC_VAR, xi, &bk, &bl, &bu));
-
-					switch (bk) {
-						case MSK_BK_FX:
-						case MSK_BK_LO:
-						case MSK_BK_RA:
-							curxx = bl;
-							break;
-						case MSK_BK_UP:
-							curxx = bu;
-							break;
-						case MSK_BK_FR:
-							curxx = 0.0;
-							break;
-						default:
-							throw msk_exception("Unexpected boundkey when loading initial solution");
+					MSKintt curval;
+					for (MSKintt j = 0; j < size; j++) {
+						errcatch( MSK_strtosk(task, const_cast<MSKCONST char*>(CHARACTER_ELT(skvalue,j).c_str()), &curval) );
+						solitem_sk[i][j] = static_cast<MSKstakeye>(curval);
 					}
-
-				} else {
-					curxx = NUMERIC_ELT(xx, xi);
 				}
-
-				MSKrealt curslx;
-				if (isEmpty(slx)) {
-					curslx = 0.0;
-				} else {
-					curslx = NUMERIC_ELT(slx, xi);
-				}
-
-				MSKrealt cursux;
-				if (isEmpty(sux)) {
-					cursux = 0.0;
-				} else {
-					cursux = NUMERIC_ELT(sux, xi);
-				}
-
-				MSKrealt cursnx;
-				if (isEmpty(snx)) {
-					cursnx = 0.0;
-				} else {
-					cursnx = NUMERIC_ELT(snx, xi);
-				}
-
-				errcatch( MSK_putsolutioni(
-						task, MSK_ACC_VAR, xi, cursoltype,
-						curskx, curxx, curslx, cursux, cursnx));
 			}
 		}
 
-		if (!anyinfocon && !anyinfovar) {
+		MSKrealt *solitem_dou[MSK_SOL_ITEM_END];
+		auto_array<MSKrealt> solitem_dou_bank[MSK_SOL_ITEM_END];
+		{
+			string name; MSKidxt size;
+
+			for (int i = MSK_SOL_ITEM_BEGIN; i < MSK_SOL_ITEM_END; ++i) {
+				MSKsoliteme itype = static_cast<MSKsoliteme>(i);
+				solitem_dou[i] = NULL;
+
+				SEXP_Handle skvalue;
+				getspecs_solitem(itype, NUMVAR, NUMCON, name, size);
+				list_seek_Numeric(skvalue, cursol, name, true);  validate_Numeric(skvalue, name, size, true);
+
+				if (!isEmpty(skvalue)) {
+					any_info = true;
+
+					if (Rf_isReal(skvalue)) {
+						solitem_dou[i] = REAL(skvalue);
+
+					} else {
+						solitem_dou_bank[i].protect(new MSKrealt[size]);
+						solitem_dou[i] = solitem_dou_bank[i];
+
+						MSKintt *curval = INTEGER(skvalue);
+						for (MSKintt j = 0; j < size; j++) {
+							solitem_dou[i][j] = static_cast<MSKrealt>(curval[j]);
+						}
+					}
+				}
+			}
+		}
+
+		if (any_info) {
+
+			errcatch( MSK_putsolution(task, cursoltype,
+				solitem_sk[MSK_SK_SOL_ITEM_SKC],
+				solitem_sk[MSK_SK_SOL_ITEM_SKX],
+				solitem_sk[MSK_SK_SOL_ITEM_SKN],
+				solitem_dou[MSK_SOL_ITEM_XC],
+				solitem_dou[MSK_SOL_ITEM_XX],
+				NULL,
+				solitem_dou[MSK_SOL_ITEM_SLC],
+				solitem_dou[MSK_SOL_ITEM_SUC],
+				solitem_dou[MSK_SOL_ITEM_SLX],
+				solitem_dou[MSK_SOL_ITEM_SUX],
+				solitem_dou[MSK_SOL_ITEM_SNX] ));
+
+		} else {
 			printwarning("The initial solution '" + name + "' was ignored.");
 		}
 	}
@@ -508,127 +587,26 @@ void get_str_parameters(SEXP_NamedVector &paramvec, MSKtask_t task)
 	}
 }
 
-bool isdef_solitem(MSKsoltypee s, MSKsoliteme v)
-{
-	switch (v)
-	{
-		// Primal variables
-		case MSK_SOL_ITEM_XC:
-		case MSK_SOL_ITEM_XX:
-			switch (s) {
-				case MSK_SOL_ITR:  return true;
-				case MSK_SOL_BAS:  return true;
-				case MSK_SOL_ITG:  return true;
-				default:
-					throw msk_exception("A solution type was not supported");
-			} break;
-
-		// Linear dual variables
-		case MSK_SOL_ITEM_SLC:
-		case MSK_SOL_ITEM_SLX:
-		case MSK_SOL_ITEM_SUC:
-		case MSK_SOL_ITEM_SUX:
-			switch (s) {
-				case MSK_SOL_ITR:  return true;
-				case MSK_SOL_BAS:  return true;
-				case MSK_SOL_ITG:  return false;
-				default:
-					throw msk_exception("A solution type was not supported");
-			} break;
-
-		// Conic dual variable
-		case MSK_SOL_ITEM_SNX:
-			switch (s) {
-				case MSK_SOL_ITR:  return true;
-				case MSK_SOL_BAS:  return false;
-				case MSK_SOL_ITG:  return false;
-				default:
-					throw msk_exception("A solution type was not supported");
-			} break;
-
-		// Ignored variables
-		case MSK_SOL_ITEM_Y:
-			return false;
-			break;
-
-		// Unsupported variables
-		default:
-			throw msk_exception("A solution item was not supported");
-	}
-}
-
-void getspecs_soltype(MSKsoltypee stype, string &name)
-{
-	switch (stype) {
-		case MSK_SOL_BAS:
-			name = "bas";
-			break;
-		case MSK_SOL_ITR:
-			name = "itr";
-			break;
-		case MSK_SOL_ITG:
-			name = "int";
-			break;
-		default:
-			throw msk_exception("A solution type was not supported");
-	}
-}
-
-void getspecs_solitem(MSKsoliteme vtype, MSKintt NUMVAR, MSKintt NUMCON, string &name, MSKidxt &size)
-{
-	switch (vtype) {
-		case MSK_SOL_ITEM_SLC:
-			name = "slc";
-			size = NUMCON;
-			break;
-		case MSK_SOL_ITEM_SLX:
-			name = "slx";
-			size = NUMVAR;
-			break;
-		case MSK_SOL_ITEM_SNX:
-			name = "snx";
-			size = NUMVAR;
-			break;
-		case MSK_SOL_ITEM_SUC:
-			name = "suc";
-			size = NUMCON;
-			break;
-		case MSK_SOL_ITEM_SUX:
-			name = "sux";
-			size = NUMVAR;
-			break;
-		case MSK_SOL_ITEM_XC:
-			name = "xc";
-			size = NUMCON;
-			break;
-		case MSK_SOL_ITEM_XX:
-			name = "xx";
-			size = NUMVAR;
-			break;
-		case MSK_SOL_ITEM_Y:
-			name = "y";
-			size = NUMCON;
-			break;
-		default:
-			throw msk_exception("A solution item was not supported");
-	}
-}
-
-void msk_getsolution(SEXP_Handle &sol, MSKtask_t task)
+void msk_getsolution(SEXP_Handle &sol, MSKtask_t task, options_type &options)
 {
   printdebug("msk_getsolution was called");
 
-	MSKintt NUMVAR, NUMCON;
-	errcatch( MSK_getnumvar(task, &NUMVAR) );
+	MSKintt NUMCON, NUMVAR, NUMCONES;
 	errcatch( MSK_getnumcon(task, &NUMCON) );
+	errcatch( MSK_getnumvar(task, &NUMVAR) );
+	errcatch( MSK_getnumcone(task, &NUMCONES) );
 
-	SEXP_NamedVector solvec;	solvec.initVEC(MSK_SOL_END - MSK_SOL_BEGIN);	sol.protect(solvec);
+	MSKnlgetspfunc nlspfunc; MSKnlgetvafunc nlvafunc;
+	errcatch( MSK_getnlfunc(task, NULL, &nlspfunc, &nlvafunc) );
+    bool isGeneralNLP = (nlspfunc != NULL) || (nlvafunc != NULL);
+
+	SEXP_NamedVector solvec;	solvec.initVEC(MSK_SOL_END);	sol.protect(solvec);
 
 	// Construct: result -> solution -> solution types
 	for (int s=MSK_SOL_BEGIN; s<MSK_SOL_END; ++s)
 	{
 		SEXP_NamedVector soltype;
-		soltype.initVEC(4 + MSK_SOL_ITEM_END);
+		soltype.initVEC(2+2+2+1 + MSK_SOL_ITEM_END);
 
 		MSKsoltypee stype = static_cast<MSKsoltypee>(s);
 		MSKintt isdef_soltype;
@@ -637,11 +615,43 @@ void msk_getsolution(SEXP_Handle &sol, MSKtask_t task)
 		if (!isdef_soltype)
 			continue;
 
-		// Add the problem status and solution status
+		// Allocate memory structures
 		MSKprostae prosta;
 		MSKsolstae solsta;
-		errcatch( MSK_getsolutionstatus(task, stype, &prosta, &solsta) );
+		auto_array<MSKstakeye> solitem_sk_bank[MSK_SK_SOL_ITEM_END];
+		{
+			string name; MSKidxt size;
+			for (int i = MSK_SK_SOL_ITEM_BEGIN; i < MSK_SK_SOL_ITEM_END; ++i) {
+				getspecs_sk_solitem(static_cast<MSK_sk_soliteme>(i), NUMVAR, NUMCON, NUMCONES, name, size);
+				solitem_sk_bank[i].protect(new MSKstakeye[size]);
+			}
+		}
+		SEXP_Vector solitem_dou_bank[MSK_SOL_ITEM_END];
+		{
+			string name; MSKidxt size;
+			for (int i = MSK_SOL_ITEM_BEGIN; i < MSK_SOL_ITEM_END; ++i) {
+				getspecs_solitem(static_cast<MSKsoliteme>(i), NUMVAR, NUMCON, name, size);
+				solitem_dou_bank[i].initREAL(size);
+			}
+		}
 
+		// Extract solution information
+		errcatch( MSK_getsolution(task, stype,
+						&prosta,
+						&solsta,
+						solitem_sk_bank[MSK_SK_SOL_ITEM_SKC],
+						solitem_sk_bank[MSK_SK_SOL_ITEM_SKX],
+						solitem_sk_bank[MSK_SK_SOL_ITEM_SKN],
+						REAL(solitem_dou_bank[MSK_SOL_ITEM_XC]),
+						REAL(solitem_dou_bank[MSK_SOL_ITEM_XX]),
+						NULL,
+						REAL(solitem_dou_bank[MSK_SOL_ITEM_SLC]),
+						REAL(solitem_dou_bank[MSK_SOL_ITEM_SUC]),
+						REAL(solitem_dou_bank[MSK_SOL_ITEM_SLX]),
+						REAL(solitem_dou_bank[MSK_SOL_ITEM_SUX]),
+						REAL(solitem_dou_bank[MSK_SOL_ITEM_SNX]) ));
+
+		// Add the problem status and solution status
 		char solsta_str[MSK_MAX_STR_LEN];
 		errcatch( MSK_solstatostr(task, solsta, solsta_str) );
 		soltype.pushback("solsta", solsta_str);
@@ -650,68 +660,134 @@ void msk_getsolution(SEXP_Handle &sol, MSKtask_t task)
 		errcatch( MSK_prostatostr(task, prosta, prosta_str) );
 		soltype.pushback("prosta", prosta_str);
 
-		// Add the constraint status keys
+		// Add the constraint, variable and conic status keys
 		{
-			auto_array<MSKstakeye> mskskc ( new MSKstakeye[NUMCON] );
+			string name; MSKidxt size; char skstrvalue[MSK_MAX_STR_LEN];
+			for (int i = MSK_SK_SOL_ITEM_BEGIN; i < MSK_SK_SOL_ITEM_END; ++i) {
+				MSK_sk_soliteme itype = static_cast<MSK_sk_soliteme>(i);
 
-			errcatch( MSK_getsolutionstatuskeyslice(task,
-									MSK_ACC_CON,	/* Request constraint status keys. */
-									stype,			/* Current solution type. */
-									0,				/* Index of first variable. */
-									NUMCON,			/* Index of last variable+1. */
-									mskskc));
+				if (itype == MSK_SK_SOL_ITEM_SKN) {
+					// No conic information in either simplex or general non-linear programming.
+					if (stype == MSK_SOL_BAS || isGeneralNLP)
+						continue;
+				}
 
-			SEXP_Vector skcvec;		skcvec.initSTR(NUMCON, false);
-			char skcname[MSK_MAX_STR_LEN];
-			for (MSKintt ci = 0; ci < NUMCON; ci++) {
-				errcatch( MSK_sktostr(task, mskskc[ci], skcname) );
-				skcvec.pushback(skcname);
+				getspecs_sk_solitem(itype, NUMVAR, NUMCON, NUMCONES, name, size);
+
+				SEXP_Vector skvec;		skvec.initSTR(size, false);
+				for (MSKintt j = 0; j < size; j++) {
+					errcatch( MSK_sktostr(task, solitem_sk_bank[i][j], skstrvalue) );
+					skvec.pushback(skstrvalue);
+				}
+				soltype.pushback(name, skvec);
 			}
-			soltype.pushback("skc", skcvec);
 		}
 
-		// Add the variable status keys
+		// Add solution variables
 		{
-			auto_array<MSKstakeye> mskskx ( new MSKstakeye[NUMVAR] );
+			string name; MSKidxt size;
+			for (int i = MSK_SOL_ITEM_BEGIN; i < MSK_SOL_ITEM_END; ++i) {
+				MSKsoliteme itype = static_cast<MSKsoliteme>(i);
 
-			errcatch( MSK_getsolutionstatuskeyslice(task,
-									MSK_ACC_VAR,	/* Request variable status keys. */
-									stype,			/* Current solution type. */
-									0,				/* Index of first variable. */
-									NUMVAR,			/* Index of last variable+1. */
-									mskskx));
+				if (!isdef_solitem( stype, itype, isGeneralNLP ))
+					continue;
 
-			SEXP_Vector skxvec;		skxvec.initSTR(NUMVAR, false);
-			char skxname[MSK_MAX_STR_LEN];
-			for (MSKintt xi = 0; xi < NUMVAR; xi++) {
-				errcatch( MSK_sktostr(task, mskskx[xi], skxname) );
-				skxvec.pushback(skxname);
+				getspecs_solitem(itype, NUMVAR, NUMCON, name, size);
+				soltype.pushback(name, solitem_dou_bank[i]);
 			}
-			soltype.pushback("skx", skxvec);
 		}
 
-		// Add solution variable slices
-		for (int v=MSK_SOL_ITEM_BEGIN; v<MSK_SOL_ITEM_END; ++v)
-		{
-			MSKsoliteme vtype = (MSKsoliteme)v;
+		// Optional: Add primal and dual objective
+		if (options.soldetail >= 1) {
+			MSKrealt primalobj, dualobj;
 
-			if (!isdef_solitem(stype, vtype))
-				continue;
+			const string PRIMAL_OBJECTIVE = "pobjval";
+			const string DUAL_OBJECTIVE = "dobjval";
+			const string PRIMAL_BOUND = "pobjbound";
 
-			string vname;
-			MSKidxt vsize;
-			getspecs_solitem(vtype, NUMVAR, NUMCON, vname, vsize);
+			switch (stype) {
+				case MSK_SOL_ITR:
+				case MSK_SOL_BAS:
+					errcatch( MSK_getdouinf(task, MSK_DINF_SOL_ITR_PRIMAL_OBJ, &primalobj) );
+					soltype.pushback(PRIMAL_OBJECTIVE, primalobj);
 
-			SEXP_Vector xxvec;	xxvec.initREAL(vsize);
-			double *pxx = REAL(xxvec);
-			errcatch( MSK_getsolutionslice(task,
-									stype, 		/* Request current solution type. */
-									vtype,		/* Which part of solution. */
-									0, 			/* Index of first variable. */
-									vsize, 		/* Index of last variable+1. */
-									pxx));
+					errcatch( MSK_getdouinf(task, MSK_DINF_SOL_ITR_DUAL_OBJ, &dualobj) );
+					soltype.pushback(DUAL_OBJECTIVE, dualobj);
+					break;
 
-			soltype.pushback(vname, xxvec);
+				case MSK_SOL_ITG:
+					errcatch( MSK_getdouinf(task, MSK_DINF_SOL_INT_PRIMAL_OBJ, &primalobj) );
+					soltype.pushback(PRIMAL_OBJECTIVE, primalobj);
+
+					MSKintt num_relax;
+					errcatch( MSK_getintinf(task, MSK_IINF_MIO_NUM_RELAX, &num_relax) );
+					if (num_relax >= 1) {
+						MSKrealt primalobjbound;
+						errcatch( MSK_getdouinf(task, MSK_DINF_MIO_OBJ_BOUND, &primalobjbound) );
+						soltype.pushback(PRIMAL_BOUND, primalobjbound);
+					} else {
+						soltype.pushback(PRIMAL_BOUND, NA_REAL);
+					}
+					break;
+				default:
+					throw msk_exception("A solution type was not supported");
+					break;
+			}
+		}
+
+		// Optional: Add maximal infeasibilities
+		if (options.soldetail >= 2) {
+			SEXP_NamedVector vecinfeas;
+			vecinfeas.initVEC(7);
+
+			MSKrealt maxpbi, maxpcni, maxpeqi, maxinti;
+			MSKrealt maxdbi, maxdcni, maxdeqi;
+			errcatch( MSK_getsolutioninf(task, stype, NULL, NULL, NULL,
+						&maxpbi,
+						&maxpcni,
+						&maxpeqi,
+						&maxinti, NULL,
+						&maxdbi,
+						&maxdcni,
+						&maxdeqi) );
+
+			const string PRIMAL_INEQUALITY 	= "pbound"; 	// MSK_DINF_SOL_ITR_MAX_PBI
+			const string PRIMAL_EQUALITY 	= "peq"; 		// MSK_DINF_SOL_ITR_MAX_PEQI
+			const string PRIMAL_CONE 		= "pcone"; 		// MSK_DINF_SOL_ITR_MAX_PCNI
+			const string DUAL_INEQUALITY 	= "dbound";  	// MSK_DINF_SOL_ITR_MAX_DBI
+			const string DUAL_EQUALITY 		= "deq"; 		// MSK_DINF_SOL_ITR_MAX_DEQI
+			const string DUAL_CONE 			= "dcone"; 		// MSK_DINF_SOL_ITR_MAX_DCNI
+			const string PRIMAL_INT 		= "int"; 		// MSK_DINF_SOL_ITR_MAX_PINTI
+
+			switch (s) {
+				case MSK_SOL_ITR:
+					vecinfeas.pushback(PRIMAL_INEQUALITY, maxpbi);
+					vecinfeas.pushback(PRIMAL_EQUALITY, maxpeqi);
+					vecinfeas.pushback(PRIMAL_CONE, maxpcni);
+					vecinfeas.pushback(DUAL_INEQUALITY, maxdbi);
+					vecinfeas.pushback(DUAL_EQUALITY, maxdeqi);
+					vecinfeas.pushback(DUAL_CONE, maxdcni);
+					break;
+
+				case MSK_SOL_BAS:
+					vecinfeas.pushback(PRIMAL_INEQUALITY, maxpbi);
+					vecinfeas.pushback(PRIMAL_EQUALITY, maxpeqi);
+					vecinfeas.pushback(DUAL_INEQUALITY, maxdbi);
+					vecinfeas.pushback(DUAL_EQUALITY, maxdeqi);
+					break;
+
+				case MSK_SOL_ITG:
+					vecinfeas.pushback(PRIMAL_INEQUALITY, maxpbi);
+					vecinfeas.pushback(PRIMAL_EQUALITY, maxpeqi);
+					vecinfeas.pushback(PRIMAL_CONE, maxpcni);
+					vecinfeas.pushback(PRIMAL_INT, maxinti);
+					break;
+
+				default:
+					throw msk_exception("A solution type was not supported");
+					break;
+			}
+			soltype.pushback("maxinfeas", vecinfeas);
 		}
 
 		string sname;
@@ -726,6 +802,75 @@ void msk_getsolution(SEXP_Handle &sol, MSKtask_t task)
 	}
 }
 
+void msk_getoptimizationinfo(SEXP_NamedVector &ret_val, Task_handle &task)
+{
+	char mskinfoname[MSK_MAX_STR_LEN];
+
+	// Integer-typed information: iinfo
+	{
+		SEXP_NamedVector vecinfo;
+		vecinfo.initVEC((MSK_IINF_END - MSK_IINF_BEGIN) + (MSK_LIINF_END - MSK_LIINF_BEGIN));
+		MSKintt msk32value;
+		MSKint64t msk64value;
+
+		// 32 bit integers
+		for (int v=MSK_IINF_BEGIN; v<MSK_IINF_END; ++v)
+		{
+			MSKiinfiteme infotype = static_cast<MSKiinfiteme>(v);
+
+			errcatch( MSK_getinfname(task, MSK_INF_INT_TYPE, infotype, mskinfoname) );
+			string infoname = mskinfoname;
+			remove_mskprefix(infoname, "MSK_IINF_");
+
+			errcatch( MSK_getintinf(task, infotype, &msk32value) );
+			vecinfo.pushback(infoname, msk32value);
+		}
+
+		// 64 bit integers
+		for (int v=MSK_LIINF_BEGIN; v<MSK_LIINF_END; ++v)
+		{
+			MSKliinfiteme infotype = static_cast<MSKliinfiteme>(v);
+
+			errcatch( MSK_getinfname(task, MSK_INF_LINT_TYPE, infotype, mskinfoname) );
+			string infoname = mskinfoname;
+			remove_mskprefix(infoname, "MSK_LIINF_");
+
+			errcatch( MSK_getlintinf(task, infotype, &msk64value) );
+			try {
+				msk32value = numeric_cast<int>(msk64value);
+			} catch (msk_exception const& ex) {
+				msk32value = NA_INTEGER;
+			}
+
+			vecinfo.pushback(infoname, msk32value);
+		}
+
+		ret_val.pushback("iinfo", vecinfo);
+	}
+
+	// Double-typed information: dinfo
+	{
+		SEXP_NamedVector vecinfo;
+		vecinfo.initVEC(MSK_DINF_END - MSK_DINF_BEGIN);
+
+		// Double-typed information
+		MSKrealt mskvalue;
+		for (int v=MSK_DINF_BEGIN; v<MSK_DINF_END; ++v)
+		{
+			MSKdinfiteme infotype = static_cast<MSKdinfiteme>(v);
+
+			errcatch( MSK_getinfname(task, MSK_INF_DOU_TYPE, infotype, mskinfoname) );
+			string infoname = mskinfoname;
+			remove_mskprefix(infoname, "MSK_DINF_");
+
+			errcatch( MSK_getdouinf(task, infotype, &mskvalue) );
+			vecinfo.pushback(infoname, mskvalue);
+		}
+
+		ret_val.pushback("dinfo", vecinfo);
+	}
+}
+
 void msk_loadproblem(Task_handle &task, problem_type &probin)
 {
 	R_len_t NUMANZ = probin.A->nnz();
